/*
	PH 09/06/08
	
	Test serial driver for the Hytec Hy8515 232/422/485
	
*/

#include <stdio.h>
#include <ctype.h>

#include <vxWorks.h>
#include <ioLib.h>
#include <iosLib.h>
#include <sioLib.h>
#include <tyLib.h>
#include <lstLib.h>
#include <errnoLib.h>

#define DEVDLS8515_C

#include "devDLS8515.h"

/* driver number assigned to this driver */

static int ty8515DrvNum = 0;

/* forward declarations */

static void   ty8515Startup(TY_8515_DEV *pTy8515Dev);
static int    ty8515Open(TY_8515_DEV *pTy8515Dev, char *name, int mode);
static STATUS ty8515Ioctl(TY_8515_DEV *pTy8515Dev, int request, int arg);
static void   ty8515Startup(TY_8515_DEV *pTy8515Dev);

/*******************************************************************************
*
* Initialise the tty driver
*
* This routine initialises the serial driver, sets up interrupt vectors, and
* performs hardware initialisation of the serial ports.
*
* RETURNS:  OK, or ERROR if the driver cannot be installed.
*
* Order: Create, Delete, Open, Close, Read, Write, Ioctl
*
*******************************************************************************/

static int ty8515Drv(void)
{
	if (ty8515DrvNum > 0) return (OK);
	
/* register driver functions */

	ty8515DrvNum = iosDrvInstall(ty8515Open, (FUNCPTR) NULL, ty8515Open, (FUNCPTR) NULL, tyRead, tyWrite, ty8515Ioctl);

	return (ty8515DrvNum == ERROR ? ERROR : OK);
}

/*******************************************************************************
*
* ty8515DevCreate - create a device for an on-board serial port
*
* This routine creates a device on a specified serial port.  Each port
* to be used should have exactly one device associated with it by calling
* this routine.
*
*	ty8515DevCreate("/ty/70/0", 0, 512, 512);
*
* RETURNS:  OK, or ERROR if the driver is not installed,
* or the device already exists.
*
*******************************************************************************/

int ty8515DevCreate(TY_8515_DEV *pTy8515Dev, char *name, int rdBufSize, int wrtBufSize)
{
	if (ty8515DrvNum == 0)
	{
		ty8515Drv();
	}
	
	if (ty8515DrvNum < 0)
	{
		errnoSet(S_ioLib_NO_DRIVER);
		return (ERROR);
	}

	if (pTy8515Dev->created == TRUE)
	{
		return (ERROR);
	}
	
	if (tyDevInit(&(pTy8515Dev->tyDev), rdBufSize, wrtBufSize, (FUNCPTR) ty8515Startup) != OK)
	{
		return (ERROR);
	}

/* mark the device as created, and add the device to the I/O system */

	pTy8515Dev->created = TRUE;
	
	pTy8515Dev->txbytes = pTy8515Dev->rxbytes = 0;
	
	if (iosDevAdd(&(pTy8515Dev->tyDev.devHdr), name, ty8515DrvNum) == ERROR)
	{
		epicsStdoutPrintf("iosDevAdd() failed\n");
		return (ERROR);
	}
	
	return (OK);
}

/*******************************************************************************
*
* Open
*
*******************************************************************************/

int ty8515Open(TY_8515_DEV *pTy8515Dev, char *name, int mode)
{
	return ((int) pTy8515Dev);
}

/*******************************************************************************
*
* Transmitter startup routine. Called whenever there are characters to write
*
*******************************************************************************/

static void ty8515Startup(TY_8515_DEV *pTy8515Dev)
{
	DLS8515Tx(pTy8515Dev);
}

/*******************************************************************************
*
* Ioctl
*
* FIOBAUDRATE		baud rate set
* FIOOPTIONS		options
* SIO_HW_OPTS_SET	set flow, data and stop bits
* SIO_HW_OPTS_GET	get flow, data and stop bits
* 
*******************************************************************************/

int ty8515Ioctl(TY_8515_DEV *pTy8515Dev, int request, int arg)
{
	int status = ERROR;
	HY8515 *pcard = (HY8515 *) pTy8515Dev->_HY8515;

	switch (request)
	{
		case FIOBAUDRATE:
		{
			status = DLS8515SetBaud(pcard, pTy8515Dev->channel, arg);
			break;
		}
		
	/* clear read and write FIFOs */
	
		case FIOFLUSH:
		{
/*
			epicsMutexLock(pcard->mutex);
			{
				UART_WRITE_SYNC(pcard->uart, pTy8515Dev->channel, UA_FCR, FCR_ENABLE | FCR_RXFRESET | FCR_TXFRESET);
				UART_WRITE_SYNC(pcard->uart, pTy8515Dev->channel, UA_FCR, FCR_ENABLE);
			}
			epicsMutexUnlock(pcard->mutex);
*/		
			status = tyIoctl(&pTy8515Dev->tyDev, request, arg);

			break;
		}
		
		case FIOWFLUSH:
		{
/*
			epicsMutexLock(pcard->mutex);
			{
				UART_WRITE_SYNC(pcard->uart, pTy8515Dev->channel, UA_FCR, FCR_ENABLE | FCR_TXFRESET);
				UART_WRITE_SYNC(pcard->uart, pTy8515Dev->channel, UA_FCR, FCR_ENABLE);
			}
			epicsMutexUnlock(pcard->mutex);
*/		
			status = tyIoctl(&pTy8515Dev->tyDev, request, arg);

			break;
		}
		
		case FIORFLUSH:
		{
/*			epicsMutexLock(pcard->mutex);
			{
				UART_WRITE_SYNC(pcard->uart, pTy8515Dev->channel, UA_FCR, FCR_ENABLE | FCR_RXFRESET);
				UART_WRITE_SYNC(pcard->uart, pTy8515Dev->channel, UA_FCR, FCR_ENABLE);
			}
			epicsMutexUnlock(pcard->mutex);
*/
			status = tyIoctl(&pTy8515Dev->tyDev, request, arg);

			break;
		}
		
		case SIO_HW_OPTS_SET:
		{
			uint8_t flow, stop, parity, data;
			
		/* parse no or hardware flow control as appropriate */

  			flow = ((arg & CLOCAL) ? 'N' : 'H');

		/* parse the wordsize (default is 8 bits) */
		
			switch (arg & CSIZE)
			{
				case CS5: data = 5; break;
				case CS6: data = 6; break;
				case CS7: data = 7; break;
				case CS8: data = 8; break;
				default:  data = 8; break;
			}

		/* parse 1 or 2 stop bits as appropriate */

   			stop = ((STOPB & arg) ? 2 : 1);

		/* check if parity checking is enabled */
		
			if (PARENB & arg)
			{
				parity = ((arg & PARODD) ? 'O' : 'E');
			}
			else
			{
 		                parity = 'N';
			}

			status = DLS8515SetOptions(pcard, pTy8515Dev->channel, data, stop, parity, flow);
			
			break;
		}

		case SIO_HW_OPTS_GET:
		{
			int opts = 0;
			
			switch (pcard->dev[pTy8515Dev->channel].data)
			{
				case 5:	opts |= CS5;
					break;
				case 6:	opts |= CS6;
					break;
				case 7:	opts |= CS7;
					break;
				case 8:	;
				
				default:
				{
					opts |= CS8;
					break;
				}
			}
			
			switch (pcard->dev[pTy8515Dev->channel].stop)
			{
				case 2:	opts |= STOPB;
					break;
				
				default:
				{
					break;
				}
			}
			
			switch (pcard->dev[pTy8515Dev->channel].parity)
			{
				case 'E':	opts |= PARENB;
						break;
				case 'O':	opts |= (PARENB | PARODD);
						break;
						
				default:
				{
					break;
				}
			}
			
			switch (pcard->dev[pTy8515Dev->channel].flow)
			{
				case 'N':	opts |= CLOCAL;
						break;
						
				default:
				{
					break;
				}
			}
			
			*(int *) arg = opts;
			
			status = OK;
			
			break;
		}

		default:
		{
			status =  tyIoctl(&pTy8515Dev->tyDev, request, arg);
			break;
		}
	}
	
	return (status);
}
